// ====================================================================
// Copyright (c) 2002-2006 CeBotics LLC.  All rights reserved.
//    This file is provided as part of a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.
//
// File:           DataPack.cpp
// Origin Date:    7/9/2002
// Platform:       Developed using MS VC++ 6.0
// Purpose:
//    This file contains the class implementation for CDataPack.
//    The CDataPack class is used to send notification datat from
//    Hardware plugin Interfaces to Devices.
// ====================================================================

// ====================================================================
// Version Information
// Ver      Date         	Eng   Description
// ---------------------------------------
// 1.00     07/09/2002		swl   Initial Version
// 1.01		11/18/2002		swl	Added support for binary data.
// 1.02		11/07/2004		swl	Fixed memcpy bug in binary AddData method
// 1.03		11/09/2006		swl	Added stdlib.h include
// ====================================================================

#include "..\stdafx.h"
#include <stdlib.h>
#include "DataPack.h"

// ====================================================================
// Method Name : CDataPack Constructor
// Parameters  : None
// Description : CDataPack Constructor
// Returns     : None
// ====================================================================
CDataPack::CDataPack()
{
   //
   // Initialize the array memory to all NULL
   memset( m_aDataNames, 0, MAX_VALUES * sizeof( char* ) );
   memset( m_aDataValues, 0, MAX_VALUES * sizeof( char* ) );
   memset( m_nDataLens, 0, MAX_VALUES * sizeof( int ) );
}

// ====================================================================
// Method Name : CDataPack Destructor
// Parameters  : None
// Description : CDataPack Destructor
// Returns     : None
// ====================================================================
CDataPack::~CDataPack()
{
   //
   // Free all memory.
   for (int nLoop = 0; m_aDataNames[ nLoop ]; nLoop++ )
   {
      free( (void*)m_aDataNames[ nLoop ] );
      free( (void*)m_aDataValues[ nLoop ] );
   }
}

// ====================================================================
// Method Name : AddData
// Parameters  : szDataName - Name of data to add.  szDataValue - Value of data
// Description : This method will add a named string data element to the pack.
// Returns     : TRUE if added successfully.  FALSE on error
// ====================================================================
BOOL CDataPack::AddData( LPCTSTR szDataName, LPCTSTR szDataValue )
{
   //
   // Make sure the same name does not already exist.
	int nLoop;
   for (nLoop = 0; m_aDataNames[ nLoop ]; nLoop++)
   {
      if (0 == strcmp( m_aDataNames[nLoop], szDataName))
      {
         return( FALSE );
      }
   }

   //
   // Make sure there is enough room to add the new data
   if (nLoop >= MAX_VALUES)
   {
      return( FALSE );
   }

   //
   // Add it to the name and value lists
   m_aDataNames[ nLoop ] = strdup( szDataName );
   m_aDataValues[ nLoop ] = strdup( szDataValue );

   return( TRUE );
}


   // ====================================================================
   // Method Name : AddData - For adding binary data
   // Parameters  : szDataName - Name of data to add.
	// 				: pDataValue - Data value
	// 				: nDataSize - Size of Data in pDataValue
   // Description : This method will add a named binary data element to the pack.
   // Returns     : TRUE if added successfully.  FALSE on error
   // ====================================================================
BOOL CDataPack::AddData( LPCTSTR szDataName, void* pDataValue, int nDataSize )
{
   //
   // Make sure the same name does not already exist.
	int nLoop;
   for (nLoop = 0; m_aDataNames[ nLoop ]; nLoop++)
   {
      if (0 == strcmp( m_aDataNames[nLoop], szDataName))
      {
         return( FALSE );
      }
   }

   //
   // Make sure there is enough room to add the new data
   if (nLoop >= MAX_VALUES)
   {
      return( FALSE );
   }

   //
   // Add it to the name and value lists
   m_aDataNames[ nLoop ] = strdup( szDataName );
   m_aDataValues[ nLoop ] = (const char*)malloc( nDataSize );
	memcpy( (void*)m_aDataValues[ nLoop ], pDataValue, nDataSize );
	m_nDataLens[ nLoop ] = nDataSize;

   return( TRUE );
}


// ====================================================================
// Method Name : GetData
// Parameters  : szDataName - Name of data to return value.
// Description : This method will return the string data that corresponds to the data name specified
// Returns     : Value of data specified by szDataName.  "" If szDataName is invalid
// ====================================================================
LPCTSTR CDataPack::GetData( LPCTSTR szDataName )
{
   //
   // Look up the data name in the data name list.
   for (int nLoop = 0; m_aDataNames[ nLoop ]; nLoop++)
   {
      if (0 == strcmp( m_aDataNames[ nLoop ], szDataName))
      {
         return( m_aDataValues[ nLoop ] );
      }
   }

   return( "" );
}

// ====================================================================
// Method Name : GetBinaryData - For reading binary data
// Parameters  : szDataName - Name of data to return value.
// 				: nDataSize - Reference that contains the size of the data returned.
// Description : This method will return the binary data that corresponds to the data name specified
// Returns     : Value of data specified by szDataName.  NULL If szDataName is invalid.
// ====================================================================
void* CDataPack::GetBinaryData( LPCTSTR szDataName, int& nDataSize )
{
   //
   // Look up the data name in the data name list.
   for (int nLoop = 0; m_aDataNames[ nLoop ]; nLoop++)
   {
      if (0 == strcmp( m_aDataNames[ nLoop ], szDataName))
      {
			nDataSize = m_nDataLens[ nLoop ];
         return( (void*)m_aDataValues[ nLoop ] );
      }
   }

	nDataSize = 0;

   return( NULL );
}
