// ====================================================================
// Copyright (c) 2002-2006 CeBotics LLC.  All rights reserved.
//    This file is provided as part of a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.
//
// File:           DataPack.h
// Origin Date:    7/9/2002
// Platform:       Developed using MS VC++ 6.0
// Purpose:
//    This file contains the class definition for CDataPack.
//    The CDataPack class is used to send notification datat from
//    Hardware plugin Interfaces to Devices.
// ====================================================================

// ====================================================================
// Version Information
// Ver      Date         	Eng   Description
// ---------------------------------------
// 1.00     07/09/2002   	swl   Initial Version
// 1.01		11/18/2002		swl	Added support for binary data.
// ====================================================================

#pragma once

#define MAX_VALUES      50

//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
// CDataPack
//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
class CDataPack
{
public:
   //
   // Basic constructor and destructor
   CDataPack();
   ~CDataPack();

   // ====================================================================
   // Method Name : AddData - For adding string data
   // Parameters  : szDataName - Name of data to add.
	// 				: szDataValue - String data value
   // Description : This method will add a named string data element to the pack.
   // Returns     : TRUE if added successfully.  FALSE on error
   // ====================================================================
   BOOL AddData( LPCTSTR szDataName, LPCTSTR szDataValue );
	
   // ====================================================================
   // Method Name : AddData - For adding binary data
   // Parameters  : szDataName - Name of data to add.
	// 				: pDataValue - Data value
	// 				: nDataSize - Size of Data in pDataValue
   // Description : This method will add a named binary data element to the pack.
   // Returns     : TRUE if added successfully.  FALSE on error
   // ====================================================================
   BOOL AddData( LPCTSTR szDataName, void* pDataValue, int nDataSize );

   // ====================================================================
   // Method Name : GetData - For reading string data
   // Parameters  : szDataName - Name of data to return value.
   // Description : This method will return the string data that corresponds to the data name specified
   // Returns     : Value of data specified by szDataName.  "" If szDataName is invalid.
   // ====================================================================
   LPCTSTR GetData( LPCTSTR szDataName );

   // ====================================================================
   // Method Name : GetBinaryData - For reading binary data
   // Parameters  : szDataName - Name of data to return value.
	// 				: nDataSize - Reference that contains the size of the data returned.
   // Description : This method will return the binary data that corresponds to the data name specified
   // Returns     : Value of data specified by szDataName.  NULL If szDataName is invalid.
   // ====================================================================
   void* GetBinaryData( LPCTSTR szDataName, int& nDataSize );

private:
   const char*    m_aDataNames[ MAX_VALUES ];
   const char*    m_aDataValues[ MAX_VALUES ];
	int				m_nDataLens[ MAX_VALUES ];		// Only valid for binary types.
};
