// ====================================================================
// Copyright (c) 2002-2006 CeBotics LLC.  All rights reserved.
//    This file is provided as part of a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.
//
// File:           InterfaceAPI.h
// Origin Date:    9/4/2002
// Platform:       Developed using MS VC++ 6.0
// Purpose:
//    This file contains common structures and enumerations that relate
//    to the InterfaceArgumentPack which is used to pass parameter data
//    from Devices to Hardware Interfaces.
// ====================================================================

// ====================================================================
// Version Information
// Ver      Date         Eng   Description
// ---------------------------------------
// 1.00     09/04/2002   swl   Initial Version
// 1.01		11/18/2002   swl   Added support for binary data.
// ====================================================================

#pragma once

//
// Valid Argument Types
enum eArgumentType
{
   atUnknown,           // Don't use
   atPropertyValue,     // The value is a property value handle.
   atStringValue,       // The value is a string (const char*) value.
	atVoidValue,			// The value is a void pointer.
   atIntValue           // The value is an integer value.
};

//
// Argument Definition.
struct InterfaceArgument
{
   eArgumentType     m_atType;            // What type is the argument?

   //
   // Union to break out the various possible types.
   union
   {
      PROPERTY_VALUE_HANDLE   m_pvhPropertyValue;
      const char*             m_szStringValue;
      int                     m_nIntValue;
		void*							m_pVoidValue;
   };
};

//
// Current version of the pack
#define INTERFACE_PACK_VERSION   1

//
// Interface Argument Pack
struct InterfaceArgumentPack
{
   int                  m_nArgumentPackVersion;       // Version that was sent from the Device.
   const char*          m_szInterfaceSignature;       // Signature of the interface the pack is targeted to.
   int                  m_nNumberOfArguments;         // Number of arguments in the array.
   InterfaceArgument*   m_aArguments;                 // Array of arguments.
};
