// ====================================================================
// Copyright (c) 2002 - 2004 Meedio, LLC.  All rights reserved.
//    This file is provided as part as a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.
//
// File          : TemplateInstance.h
// Origin Date   : 09/7/2002
// Purpose       : This file contains the implementation of the
//                   CTemplateHardwareInstance class for providing
//                   all of the functionality required for a basic
//                   Hardware Interface Plugin.
// ====================================================================

// ====================================================================
// Version Information
// Ver      Date         Eng   Description
// ---------------------------------------
// 1.00     09/7/2002    swl    Initial Version
// 1.01     12/30/2002   swl    Added Query Method to support HBQueryInstance()
// ====================================================================

#pragma once

#include "..\Common\HardwareInstance.h"
#include "..\..\ahscript.h"

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
// CTemplateHardwareInstance - Sample Hardware Interface Plugin
//       instance class.  This class is derived from CHardwareInstance
//       to inherit all helper functions.
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
class CTemplateHardwareInstance : public CHardwareInstance
{
public:
   CTemplateHardwareInstance( HARDWARE_INSTANCE_HANDLE hInstance, CallBackInfo* pCallBackInfo );

   //
   // Derived from virtual methods in CHardwareInstance class to service
   // standard server requests.
   BOOL Init();
   void Terminate();
   BOOL RegisterProperties();
   BOOL UpdatePropertyValue( const char* szPropertyName, const char* szNewValue );
   BOOL PropertyButtonClick( int nButtonCallbackID );
   BOOL Query( const char* szQuery, void* pResponse );
	HARDWARE_RC InterfaceCall( InterfaceArgumentPack* pPack );

   //
   // Plugin specific interfaces.
   HARDWARE_RC SendX10( InterfaceArgumentPack* pPack );
   //HARDWARE_RC SendLevitonDim( InterfaceArgumentPack* pPack );
   //HARDWARE_RC SendPresetDim( InterfaceArgumentPack* pPack );

   //HARDWARE_RC DisplayMessageBox( InterfaceArgumentPack* pPack );
   //HARDWARE_RC Beep( InterfaceArgumentPack* pPack );

private:
    BOOL  m_bEnabled;
    IActiveHome* pActiveHome;
	DWORD	m_dwStartCount;
	HARDWARE_INSTANCE_HANDLE hInstanceHandle;
};

//
// Collection of all instances of CTemplateHardwareInstance
typedef  std::map< HARDWARE_INSTANCE_HANDLE, CTemplateHardwareInstance* >     HARDWARE_INSTANCE_MAP;

