#pragma once

//
// Disable warning about debug symbols
#pragma warning(disable:4786)
#include <map>

#include "HardwareAPI.h"

//
// Forward definitions
struct InterfaceArgument;
struct InterfaceArgumentPack;
class CDataPack;

///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// CHardwareInstance
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
class CHardwareInstance
{
public:
   virtual ~CHardwareInstance() {};

   //
   // Derived classes have the option of deriving these methods.  These
   // methods map to exports that are called by the server.
   virtual BOOL Init() { return( TRUE ); }
   virtual void Terminate() {};
   virtual BOOL RegisterProperties() { return( TRUE ); }
   virtual BOOL UpdatePropertyValue( const char* szPropertyName, const char* szNewValue ) { return( TRUE ); }
   virtual BOOL PropertyButtonClick( int nButtonCallbackID ) { return( TRUE ); }
   virtual HARDWARE_RC HBInterface( const char* szInterfaceName, InterfaceArgumentPack* pPack   ) { return( hrcFailureAbort ); }

   inline HARDWARE_INSTANCE_HANDLE GetInstanceHandle() { return( m_hInstance ); }

   ///////////////////////////////////////////////////////////////////////
   ///////////////////////////////////////////////////////////////////////
   // Call back helper wrappers.  See HardwareAPI.h for description
   // of the methods and their parameters.
   ///////////////////////////////////////////////////////////////////////
   ///////////////////////////////////////////////////////////////////////
   BOOL           RegisterHardwareModuleProperty( const char* szPropertyName, P_TYPE pType, const char** aValues, const char* szDefaultValue, BOOL bShowInMainConfiguration, const char* szWizardText = NULL );
   BOOL           SetPropertyLimits( const char* szPropertyName, int nLowLimit, int nHighLimit );
   BOOL           RegisterHardwareModuleButton( int nButtonCallbackID, const char* szButtonHeading, const char* szButtonText );
   const char*    GetPropertyNameFromHandle( PROPERTY_VALUE_HANDLE hPropertyValue );
   const char*    GetPropertyValueFromName( const char* szPropertyName );
   const char*    GetPropertyValueFromHandle( PROPERTY_VALUE_HANDLE hPropertyValue );
   BOOL           GetPropertyInfoAtIndex( int nIndex, char* szPropertyName, char* szPropertyValue );
   BOOL           SetPropertyValueFromName( const char* szPropertyName, const char* szPropertyValue );
   BOOL           SetPropertyNameAndValueAtIndex( int nIndex, const char* szPropertyName, const char* szPropertyValue );
   BOOL           DeletePropertyValueByNameAndValue( const char* szPropertyName, const char* szPropertyValue );
   BOOL           DeletePropertyValueByIndex( int nIndex );
   void           NotifySubscribedDevices( LPCTSTR szListName, LPCTSTR szFilter, CDataPack* pDataPack );
   BOOL           GetIRCodes( IRCode* paCodes, DWORD* pdwBufferSize );
   BOOL           GetIRCodeFromPosition( unsigned int usIRPosition, IRCode* pCode );
   BOOL           AddOrUpdateIRCode( IRCode* pCode, BOOL bNewCode );
   BOOL           RemoveIRCode( IRCode* pCode );
   BOOL           IsRemoteControlDescriptionUnique( const char* szRemoteDescription );
   BOOL           GetRemoteControlList( RemoteControl* paRetRemotes, DWORD* pdwBufferSize );
   BOOL           AddRemoteControl( const char* szRemoteDescription, int nFrequency, const char* szManufacturer, const char* szModel );
   void           TraceMessage( TRC_TYPE ttType, const char* szMessage, ... );


   ///////////////////////////////////////////////////////////////////////
   ///////////////////////////////////////////////////////////////////////
   // Other helper methods
   ///////////////////////////////////////////////////////////////////////
   ///////////////////////////////////////////////////////////////////////
   const char*    CharToDump( const unsigned char* szData, int nDataLen, char* szReturnBuffer, int nReturnBufferLen );
   char*          GetLastErrorAsString();

   int            DoesDeviceExist( const char* szDeviceName, BOOL bExactMatch, const char* szDeviceDescription, const char* szPropertyName, const char* szPropertyValue, char** szActualDeviceDescription = NULL );
   int            CreateDevice( const char* szDeviceName, const char* szDeviceDescription, const char* szDeviceComment, HARDWARE_INSTANCE_HANDLE hInstance );
   BOOL           SetDeviceProperty( int nDeviceID, const char* szPropertyName, const char* szPropertyValue );

protected:
   CHardwareInstance( HARDWARE_INSTANCE_HANDLE hInstance, CallBackInfo* pCallBackInfo );

   //
   // Helper functions to parse InterfaceArgumentPack arguments.
   const char*    TextFromInterfaceArgument( InterfaceArgument* pArgument );
   int            IntFromInterfaceArgument( InterfaceArgument* pArgument );
   void*          VoidFromInterfaceArgument( InterfaceArgument* pArgument );

   BOOL           CheckForServerVersion( int nMajor, int nMinor, int nSubMinor );


   HARDWARE_INSTANCE_HANDLE   m_hInstance;
   CallBackInfo*              m_pCallBackInfo;
};


// ====================================================================
// Class Name  : CMFCStateSwitcher
// Description : This class helps to manage the MFC thread/module
//                state of MFC DLLs.  It is used by the CHardwareInstance
//                class before calling back into the server.
// ====================================================================
#ifdef __AFX_H__
class CMFCStateSwitcher
{
public:
   CMFCStateSwitcher() { m_pDLLModuleState = AfxSetModuleState( AfxGetThreadState()->m_pPrevModuleState );  }
   ~CMFCStateSwitcher() { AfxSetModuleState( m_pDLLModuleState ); }

   AFX_MODULE_STATE*    m_pDLLModuleState;
};

#else
class CMFCStateSwitcher
{
public:
   CMFCStateSwitcher() {};
};

#endif
